/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.felix.upnp.basedriver.controller;

/* 
* @author <a href="mailto:dev@felix.apache.org">Felix Project Team</a>
*/
public interface DevicesInfo{
    /**
     * 
     * Allow you to get the URL of the XML description of
     * a device specified by UUID. 
     * 
     * @param udn the UUID that identify a device
     * @return a String representing the URL pointing to the description of the Device
     */
    public String getLocationURL(String udn);
    
    /**
     * 
     * Allow you to get the URL of the XML description of
     * a service specified by the ServiceId and the UUID of the device that 
     * contains the service 
     * 
     * @param udn the UUID of the device that contain the service
     * @param serviceId the ServiceId of the service
     * @return a String representing the URL pointing to the description of the Device
     */
    public String getSCPDURL(String udn,String serviceId);  
    
    /**
     * Allow you to get the absolute URL of a link that is contained in a device. 
     * For example the presentation page of a device might be relative to the BASE URL 
     * declared in the device description.
     * 
     * @param udn the UUID of the UPnP Device 
     * @param link the relative link that you want to resolve
     * @return a String representing the absolute URL of the resource specified by the link
     */
    public String resolveRelativeUrl(String udn, String link);
    
}
